<?xml version = "1.0" encoding = "UTF-8"?>

<!--
  xmLP version 1.0.
  Copyright (C) 2000-2002 Anthony B. Coates & Reuters.
  Released under the Lesser GNU Public License, version 2.1.
-->

<xsl:stylesheet version = "1.0" xmlns = "http://www.w3.org/1999/xhtml" xmlns:xsl = "http://www.w3.org/1999/XSL/Transform" xmlns:lp = "http://www.xmLP.org/2001/ns/xmLP/">
	<xsl:import href = "xmLPcommon.xsl"/>
	<xsl:output method = "xml" indent = "yes" version = "1.0" doctype-public = "-//W3C//DTD XHTML 1.1//EN" doctype-system = "dtd/xhtml11-flat.dtd"/>
	<!-- Indent string (two spaces). -->
	<xsl:variable name = "tempIndent"><![CDATA[x  x]]></xsl:variable>
	<xsl:param name = "XmlIndent" select = "substring($tempIndent, 2, string-length($tempIndent)-2)"/>
	<!-- This is the template which is called first. -->
	<xsl:template match = "/">
		<xsl:apply-templates/>
	</xsl:template>
	<!--By default, document content is copied unchanged. -->
	
	<!--This template is designed to exclude comments from the DTD,
but in the process it excludes any comments which
are outside of the top-level element.-->
	<xsl:template match = "*|text()|*/comment()|processing-instruction()">
		<xsl:copy>
			<xsl:for-each select = "@*">
				<xsl:copy/>
			</xsl:for-each>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	<!-- **** "xmLP" support **** -->
	
	<!--Weaves an invoked macro. -->
	<xsl:template name = "invokeMacro">
		<xsl:param name = "macroName"/>
		<xsl:param name = "indent"/>
		<xsl:param name = "isForceInline">false</xsl:param>
		<xsl:variable name = "position" select = "lp:index"/>
		<xsl:variable name = "macroId">
			<xsl:call-template name = "getMacroId">
				<xsl:with-param name = "position">
					<xsl:value-of select = "$position" disable-output-escaping = "no"/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		<a href = "#{$macroId}">
			<xsl:value-of select = "$macroName"/>
			<xsl:text>[</xsl:text>
			<xsl:value-of select = "$position"/>
			<xsl:text>]</xsl:text>
		</a>
	</xsl:template>
	<!--This template renders an "lp:file", which corresponds to an output file. -->
	<xsl:template match = "lp:file">
		<div class = "xmlp-file-def">
			<xsl:variable name = "position" select = "lp:index"/>
			<xsl:variable name = "fileID">
				<xsl:call-template name = "getFileId">
					<xsl:with-param name = "position">
						<xsl:value-of select = "$position" disable-output-escaping = "no"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:variable>
			<a id = "{$fileID}"/>
			<dl>
				<dt>
					<span class = "xmlp-file-name">
						<xsl:text>File[</xsl:text>
						<xsl:value-of select = "$position" disable-output-escaping = "no"/>
						<xsl:text><![CDATA[]: ]]></xsl:text>
						<xsl:value-of select = "@lp:filename" disable-output-escaping = "no"/>
						<xsl:text><![CDATA[ =]]></xsl:text>
					</span>
				</dt>
				<dd>
					<pre>
						<xsl:text disable-output-escaping = "no"/>
						<xsl:call-template name = "processContent">
							<xsl:with-param name = "isForceInline">true</xsl:with-param>
						</xsl:call-template>
					</pre>
				</dd>
			</dl>
		</div>
	</xsl:template>
	<!--This template renders an "lp:macro", which is a text or balanced XML fragment. -->
	<xsl:template match = "lp:macro">
		<xsl:variable name = "class">
			<xsl:choose>
				<xsl:when test = "lp:invokedInMacro or lp:invokedInFile">
					<xsl:text>xmlp-macro-def</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>xmlp-z-macro-def</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<div class = "{$class}">
			<xsl:variable name = "position" select = "lp:index"/>
			<xsl:variable name = "macroID">
				<xsl:call-template name = "getMacroId">
					<xsl:with-param name = "position">
						<xsl:value-of select = "$position"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:variable>
			<a id = "{$macroID}"/>
			<dl>
				<dt>
					<span class = "xmlp-macro-name">
						<xsl:value-of select = "lp:name"/>
						<xsl:text>[</xsl:text>
						<xsl:value-of select = "$position"/>
						<xsl:text>]=</xsl:text>
					</span>
				</dt>
				<dd>
					<pre>
						<xsl:text/>
						<xsl:call-template name = "processContent">
							<xsl:with-param name = "isForceInline">true</xsl:with-param>
						</xsl:call-template>
					</pre>
				</dd>
			</dl>
			<xsl:choose>
				<xsl:when test = "lp:invokedInMacro or lp:invokedInFile">
					<h5>
						<xsl:if test = "lp:invokedInMacro">
							<xsl:text>This macro is invoked in macros:</xsl:text>
							<xsl:for-each select = "lp:invokedInMacro">
								<xsl:text><![CDATA[ ]]></xsl:text>
								<xsl:element name = "a">
									<xsl:attribute name = "href">
										<xsl:text>#</xsl:text>
										<xsl:call-template name = "getMacroId">
											<xsl:with-param name = "position">
												<xsl:value-of select = "."/>
											</xsl:with-param>
										</xsl:call-template>
									</xsl:attribute>
									<xsl:value-of select = "."/>
								</xsl:element>
							</xsl:for-each>
						</xsl:if>
						<xsl:if test = "lp:invokedInFile">
							<xsl:if test = "lp:invokedInMacro">
								<br class = "br"/>
							</xsl:if>
							<xsl:text>This macro is invoked in files:</xsl:text>
							<xsl:for-each select = "lp:invokedInFile">
								<xsl:text><![CDATA[ ]]></xsl:text>
								<xsl:element name = "a">
									<xsl:attribute name = "href">
										<xsl:text>#</xsl:text>
										<xsl:call-template name = "getFileId">
											<xsl:with-param name = "position">
												<xsl:value-of select = "."/>
											</xsl:with-param>
										</xsl:call-template>
									</xsl:attribute>
									<xsl:value-of select = "."/>
								</xsl:element>
							</xsl:for-each>
						</xsl:if>
					</h5>
				</xsl:when>
				<xsl:otherwise>
					<h5>This macro is NEVER invoked.</h5>
				</xsl:otherwise>
			</xsl:choose>
		</div>
	</xsl:template>
	<!--Returns a macro ID for the given index-->
	<xsl:template name = "getMacroId">
		<xsl:param name = "position"/>
		<xsl:text>xmLPmacro</xsl:text>
		<xsl:value-of select = "$position"/>
		<xsl:text>orcamPLmx</xsl:text>
	</xsl:template>
	<!--Returns a file ID for the given index-->
	<xsl:template name = "getFileId">
		<xsl:param name = "position"/>
		<xsl:text>xmLPfile</xsl:text>
		<xsl:value-of select = "$position"/>
		<xsl:text>elifPLmx</xsl:text>
	</xsl:template>
	<!--Writes out XML content textually.-->
	
	<!--This version is modified for weaving.-->
	<xsl:template name = "writeXml">
		<xsl:param name = "isTopLevel" select = "true"/>
		<xsl:param name = "indent"/>
		<xsl:text disable-output-escaping = "yes"><![CDATA[<span class="xmlp-punctuation">&lt;</span><span class="xmlp-element-name">]]></xsl:text>
		<xsl:value-of select = "name()"/>
		<xsl:text disable-output-escaping = "yes"><![CDATA[</span>]]></xsl:text>
		<xsl:for-each select = "@*">
			<xsl:text disable-output-escaping = "yes"><![CDATA[ <span class="xmlp-attribute-name">]]></xsl:text>
			<xsl:value-of select = "name()"/>
			<xsl:text disable-output-escaping = "yes"><![CDATA[</span><span class="xmlp-punctuation">="</span>]]></xsl:text>
			<xsl:value-of select = "."/>
			<xsl:text disable-output-escaping = "yes"><![CDATA[<span class="xmlp-punctuation">"</span>]]></xsl:text>
		</xsl:for-each>
		<xsl:if test = "parent::lp:xml[parent::lp:file or parent::lp:macro]">
			<xsl:for-each select = "../../lp:namespace">
				<xsl:text disable-output-escaping = "yes"><![CDATA[ <span class="xmlp-namespace-name">xmlns]]></xsl:text>
				<xsl:if test = "@lp:prefix">
					<xsl:text>:</xsl:text>
				</xsl:if>
				<xsl:value-of select = "@lp:prefix"/>
				<xsl:text disable-output-escaping = "yes"><![CDATA[</span><span class="xmlp-punctuation">="</span>]]></xsl:text>
				<xsl:value-of select = "@lp:value"/>
				<xsl:text disable-output-escaping = "yes"><![CDATA[<span class="xmlp-punctuation">"</span>]]></xsl:text>
			</xsl:for-each>
		</xsl:if>
		<xsl:if test = "not(node())">
			<xsl:text disable-output-escaping = "yes"><![CDATA[<span class="xmlp-punctuation">/&gt;</span>]]></xsl:text>
		</xsl:if>
		<xsl:if test = "node()">
			<xsl:text disable-output-escaping = "yes"><![CDATA[<span class="xmlp-punctuation">&gt;</span>]]></xsl:text>
			<xsl:for-each select = "*|text()">
				<xsl:call-template name = "tangleXml">
					<xsl:with-param name = "indent" select = "$indent"/>
					<xsl:with-param name = "isForceInline">false</xsl:with-param>
				</xsl:call-template>
			</xsl:for-each>
			<xsl:if test = "not(normalize-space(text()))">
				<xsl:text disable-output-escaping = "yes"><![CDATA[
]]></xsl:text>
				<xsl:value-of select = "$indent"/>
			</xsl:if>
			<xsl:text disable-output-escaping = "yes"><![CDATA[<span class="xmlp-punctuation">&lt;/</span><span class="xmlp-element-name">]]></xsl:text>
			<xsl:value-of select = "name()"/>
			<xsl:text disable-output-escaping = "yes"><![CDATA[</span><span class="xmlp-punctuation">&gt;</span>]]></xsl:text>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>