<?xml version = "1.0" encoding = "UTF-8"?>

<!--
  xmLP version 1.0.
  Copyright (C) 2000-2002 Anthony B. Coates & Reuters.
  Released under the Lesser GNU Public License, version 2.1.
-->

<!--The "invokeMacro" and "writeXml" templates must be supplied by the importing template.-->
<xsl:stylesheet version = "1.0" xmlns:xsl = "http://www.w3.org/1999/XSL/Transform" xmlns:lp = "http://www.xmLP.org/2001/ns/xmLP/">
	<xsl:import href = "xmLPvalidate.xsl"/>
	<!--Processes macro/file content.-->
	<xsl:template name = "processContent">
		<xsl:param name = "indent"/>
		<xsl:param name = "isForceInline">false</xsl:param>
		<xsl:text/>
		<xsl:for-each select = "lp:text | lp:xml">
			<xsl:if test = "self::lp:text">
				<xsl:call-template name = "tangleText"/>
			</xsl:if>
			<xsl:if test = "self::lp:xml">
				<xsl:call-template name = "tangleXml">
					<xsl:with-param name = "indent" select = "$indent"/>
					<xsl:with-param name = "isForceInline" select = "$isForceInline='true' or child::lp:text and (child::lp:xml/text() and normalize-space(child::lp:xml/text()))"/>
				</xsl:call-template>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	<!-- Tangler for text macros. -->
	<xsl:template name = "tangleText">
		<xsl:for-each select = "*|text()">
			<xsl:choose>
				<xsl:when test = "self::text()">
					<xsl:value-of select = "."/>
				</xsl:when>
				<xsl:when test = "self::lp:invoke">
					<xsl:call-template name = "invokeMacro">
						<xsl:with-param name = "macroName" select = "normalize-space(lp:name)"/>
					</xsl:call-template>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	<!-- Tangler for balanced XML macros. -->
	
	<!-- As XSL-T tends to swallow up namespace declarations, -->
	
	<!-- and this script uses the "text" output mode, -->
	
	<!-- it is necessary to add namespace declarations manually. -->
	
	<!--The "writeXml" template must be supplied by the importing template.-->
	<xsl:template name = "tangleXml">
		<xsl:param name = "indent"/>
		<xsl:param name = "isForceInline">false</xsl:param>
		<xsl:text/>
		<xsl:choose>
			<xsl:when test = "self::lp:xml">
				<xsl:for-each select = "*|text()">
					<xsl:call-template name = "tangleXml">
						<xsl:with-param name = "indent" select = "$indent"/>
						<!--**** value here was "false", now "$isForceInline" ****-->
						<xsl:with-param name = "isForceInline" select = "$isForceInline"/>
					</xsl:call-template>
				</xsl:for-each>
			</xsl:when>
			<xsl:when test = "self::text()">
				<xsl:if test = "normalize-space()">
					<xsl:value-of select = "." disable-output-escaping = "no"/>
				</xsl:if>
			</xsl:when>
			<xsl:when test = "self::lp:invoke">
				<xsl:variable name = "isMixedContent" select = "(preceding-sibling::text() and normalize-space(preceding-sibling::text())) or (following-sibling::text() and normalize-space(following-sibling::text()))"/>
				<xsl:call-template name = "invokeMacro">
					<xsl:with-param name = "macroName" select = "normalize-space(lp:name)"/>
					<xsl:with-param name = "indent" select = "$indent"/>
					<xsl:with-param name = "isForceInline" select = "$isForceInline='true' or $isMixedContent"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test = "self::*">
				<xsl:variable name = "isMixedContent" select = "(preceding-sibling::text() and normalize-space(preceding-sibling::text())) or (following-sibling::text() and normalize-space(following-sibling::text()))"/>
				<xsl:variable name = "isTopLevel" select = "parent::lp:xml and ancestor::lp:file"/>
				<xsl:variable name = "isInline" select = "$isForceInline='true' or $isMixedContent or $isTopLevel"/>
				<xsl:choose>
					<xsl:when test = "$isInline">
						<xsl:call-template name = "writeXml">
							<xsl:with-param name = "isTopLevel" select = "$isTopLevel"/>
							<xsl:with-param name = "indent" select = "$indent"/>
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:variable name = "newIndent">
							<xsl:text/>
							<xsl:value-of select = "$indent"/>
							<xsl:value-of select = "$XmlIndent"/>
						</xsl:variable>
						<xsl:text><![CDATA[
]]></xsl:text>
						<xsl:value-of select = "$newIndent"/>
						<xsl:call-template name = "tangleXml">
							<xsl:with-param name = "indent" select = "$newIndent"/>
							<xsl:with-param name = "isForceInline">true</xsl:with-param>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
		</xsl:choose>
	</xsl:template>
</xsl:stylesheet>
<!-- End of xmLPcommon.xsl -->
