<?xml version = "1.0" encoding = "UTF-8"?>

<!--
  xmLP version 1.0.
  Copyright (C) 2000-2002 Anthony B. Coates & Reuters.
  Released under the Lesser GNU Public License, version 2.1.
-->

<xsl:stylesheet version = "1.1" xmlns:xsl = "http://www.w3.org/1999/XSL/Transform" xmlns:lp = "http://www.xmLP.org/2001/ns/xmLP/">
	<xsl:import href = "xmLPcommon.xsl"/>
	<xsl:output method = "text" indent = "no"/>
	<!-- Indent string (a tab). -->
	<xsl:variable name = "tempIndent"><![CDATA[x	x]]></xsl:variable>
	<xsl:param name = "XmlIndent" select = "substring($tempIndent, 2, string-length($tempIndent)-2)"/>
	<!--
		Default rule:
		Do not tangle anything unless told to explicitly.
	-->
	<xsl:template match = "*|@*|text()|comment()"/>
	<!--Writes the content to a file.  Override if your XSL-T engine does not support xsl:document.-->
	<xsl:template name = "writeToFile">
		<xsl:param name = "filename"/>
		<xsl:document href = "{$filename}">
			<xsl:call-template name = "writeFileContent"/>
		</xsl:document>
	</xsl:template>
	<!--Writes the file content.-->
	<xsl:template name = "writeFileContent">
		<xsl:for-each select = "lp:text | lp:xml">
			<xsl:if test = "self::lp:text">
				<xsl:call-template name = "tangleText"/>
			</xsl:if>
			<xsl:if test = "self::lp:xml">
				<xsl:call-template name = "tangleXml"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	<!--
		Locate the files to be tangled, and tangle them.
	-->
	<xsl:template match = "/">
		<xsl:for-each select = "//lp:file">
			<xsl:call-template name = "validateFile"/>
			<xsl:variable name = "filename" select = "@lp:filename"/>
			<xsl:message>
				<xsl:text>Tangling: </xsl:text>
				<xsl:value-of select = "$filename"/>
				<xsl:text><![CDATA[ ...
]]></xsl:text>
			</xsl:message>
			<xsl:call-template name = "writeToFile">
				<xsl:with-param name = "filename" select = "$filename"/>
			</xsl:call-template>
		</xsl:for-each>
		<xsl:message>
			<xsl:text><![CDATA[Done.
]]></xsl:text>
		</xsl:message>
	</xsl:template>
	<!-- Expands an invoked macro. -->
	<xsl:template name = "invokeMacro">
		<xsl:param name = "macroName"/>
		<xsl:param name = "indent"/>
		<xsl:param name = "isForceInline">false</xsl:param>
		<xsl:text disable-output-escaping = "no"/>
		<xsl:for-each select = "//lp:macro[normalize-space(lp:name) = $macroName]">
			<xsl:call-template name = "validateMacro"/>
			<xsl:call-template name = "processContent">
				<xsl:with-param name = "indent" select = "$indent"/>
				<xsl:with-param name = "isForceInline" select = "$isForceInline"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	<!--Writes out XML content textually.-->
	
	<!--This version is suitable for tangling rather than weaving.-->
	<xsl:template name = "writeXml">
		<xsl:param name = "isTopLevel" select = "true"/>
		<xsl:param name = "indent"/>
		<xsl:text>&lt;</xsl:text>
		<xsl:value-of select = "name()" disable-output-escaping = "no"/>
		<xsl:for-each select = "@*">
			<xsl:text><![CDATA[ ]]></xsl:text>
			<xsl:value-of select = "name()" disable-output-escaping = "no"/>
			<xsl:text disable-output-escaping = "no">="</xsl:text>
			<xsl:value-of select = "." disable-output-escaping = "no"/>
			<xsl:text disable-output-escaping = "no">"</xsl:text>
		</xsl:for-each>
		<xsl:if test = "parent::lp:xml[parent::lp:file or parent::lp:macro]">
			<xsl:for-each select = "../../lp:namespace">
				<xsl:text><![CDATA[ xmlns]]></xsl:text>
				<xsl:if test = "@lp:prefix">
					<xsl:text disable-output-escaping = "no">:</xsl:text>
				</xsl:if>
				<xsl:value-of select = "@lp:prefix" disable-output-escaping = "no"/>
				<xsl:text disable-output-escaping = "no">="</xsl:text>
				<xsl:value-of select = "@lp:value" disable-output-escaping = "no"/>
				<xsl:text disable-output-escaping = "no">"</xsl:text>
			</xsl:for-each>
		</xsl:if>
		<xsl:if test = "not(node())">
			<xsl:text>/></xsl:text>
		</xsl:if>
		<xsl:if test = "node()">
			<xsl:text>></xsl:text>
			<xsl:for-each select = "*|text()">
				<xsl:call-template name = "tangleXml">
					<xsl:with-param name = "indent" select = "$indent"/>
					<xsl:with-param name = "isForceInline">false</xsl:with-param>
				</xsl:call-template>
			</xsl:for-each>
			<xsl:if test = "not(normalize-space(text()))">
				<xsl:text><![CDATA[
]]></xsl:text>
				<xsl:value-of select = "$indent" disable-output-escaping = "no"/>
			</xsl:if>
			<xsl:text>&lt;/</xsl:text>
			<xsl:value-of select = "name()" disable-output-escaping = "no"/>
			<xsl:text>></xsl:text>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
<!-- End of xmLPtangle.xsl -->
