<?xml version = "1.0" encoding = "UTF-8"?>

<!--
  xmLP version 1.0.
  Copyright (C) 2000-2002 Anthony B. Coates & Reuters.
  Released under the Lesser GNU Public License, version 2.1.
-->

<xsl:stylesheet version = "1.0" xmlns:xsl = "http://www.w3.org/1999/XSL/Transform" xmlns:lp = "http://www.xmLP.org/2001/ns/xmLP/">
	<!-- **** "xmLP" support **** -->
	
	<!-- This template validates an "lp:file", which corresponds to an output file. -->
	<xsl:template name = "validateFile">
		<!-- Check that the output filename is unique. -->
		<xsl:variable name = "filename">
			<xsl:value-of select = "@lp:filename"/>
		</xsl:variable>
		<xsl:variable name = "numSameFilename">
			<xsl:value-of select = "count(//lp:file[@lp:filename = $filename])"/>
		</xsl:variable>
		<xsl:if test = "$numSameFilename > 1">
			<xsl:message terminate = "yes">
				<xsl:text>!! xmLP error: file defined</xsl:text>
				<xsl:value-of select = "$numSameFilename"/>
				<xsl:text> times : </xsl:text>
				<xsl:value-of select = "$filename"/>
			</xsl:message>
		</xsl:if>
	</xsl:template>
	<!-- This template validates an "lp:macro", which is a text or balanced XML fragment. -->
	<xsl:template name = "validateMacro">
		<!-- Check that the macro is consistently and sensibly defined. -->
		<xsl:variable name = "macroName" select = "normalize-space(lp:name)"/>
		<xsl:if test = "string-length($macroName) &lt; 1">
			<xsl:message terminate = "yes">
				<xsl:text>!! xmLP error: macro did not have a non-empty name</xsl:text>
			</xsl:message>
		</xsl:if>
		<xsl:variable name = "macroDefinitions" select = "//lp:macro/lp:name[normalize-space(text())=$macroName]"/>
		<xsl:variable name = "numDefinitions" select = "count($macroDefinitions)"/>
		<xsl:if test = "@lp:final = 'true' and numDefinitions > 1">
			<xsl:message terminate = "yes">
				<xsl:text>!! xmLP error: a 'final' macro may not have multiple definitions: </xsl:text>
				<xsl:value-of select = "lp:name"/>
			</xsl:message>
		</xsl:if>
		<xsl:if test = "@lp:final = 'false' and numDefinitions &lt; 2">
			<xsl:message terminate = "no">
				<xsl:text>== xmLP remark: a macro did not have expected multiple definitions: </xsl:text>
				<xsl:value-of select = "lp:name"/>
			</xsl:message>
		</xsl:if>
		<xsl:variable name = "macroInvocations" select = "//lp:macro//lp:invoke/lp:name[normalize-space(text())=$macroName]"/>
		<xsl:variable name = "fileInvocations" select = "//lp:file//lp:invoke/lp:name[normalize-space(text())=$macroName]"/>
		<xsl:variable name = "numInvocations" select = "count($macroInvocations) + count($fileInvocations)"/>
		<xsl:if test = "@lp:usage = 'none' and $numInvocations != 0">
			<xsl:message terminate = "yes">
				<xsl:text>!! xmLP error: a macro with 'none' usage was invoked </xsl:text>
				<xsl:value-of select = "$numInvocations"/>
				<xsl:text> time(s): </xsl:text>
				<xsl:value-of select = "lp:name"/>
			</xsl:message>
		</xsl:if>
		<xsl:if test = "@lp:usage = 'once' and $numInvocations != 1">
			<xsl:message terminate = "yes">
				<xsl:text>!! xmLP error: a macro with 'once' usage was invoked </xsl:text>
				<xsl:value-of select = "$numInvocations"/>
				<xsl:text> times: </xsl:text>
				<xsl:value-of select = "lp:name"/>
			</xsl:message>
		</xsl:if>
		<xsl:if test = "@lp:usage = 'multiple' and $numInvocations &lt; 2">
			<xsl:message terminate = "no">
				<xsl:text>== xmLP remark: a macro with 'multiple' usage was only invoked </xsl:text>
				<xsl:value-of select = "$numInvocations"/>
				<xsl:text> time(s): </xsl:text>
				<xsl:value-of select = "lp:name"/>
			</xsl:message>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>