<?xml version = "1.0" encoding = "UTF-8"?>

<!--
  xmLP version 1.0.
  Copyright (C) 2000-2002 Anthony B. Coates & Reuters.
  Released under the Lesser GNU Public License, version 2.1.
-->

<xsl:stylesheet version = "1.0" xmlns:xsl = "http://www.w3.org/1999/XSL/Transform" xmlns:lp = "http://www.xmLP.org/2001/ns/xmLP/">
	<xsl:import href = "xmLPvalidate.xsl"/>
	<!-- This is the template which is called first. -->
	<xsl:template match = "/">
		<xsl:apply-templates/>
	</xsl:template>
	<!--By default, document content is copied unchanged. -->
	
	<!--This template is designed to exclude comments from the DTD,
but in the process it excludes any comments which
are outside of the top-level element.-->
	<xsl:template match = "*|text()|*/comment()|processing-instruction()">
		<xsl:copy>
			<xsl:for-each select = "@*">
				<xsl:copy/>
			</xsl:for-each>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	<!-- **** "xmLP" support **** -->
	
	<!-- This template renders an "lp:file", which corresponds to an output file. -->
	<xsl:template match = "lp:file">
		<xsl:call-template name = "validateFile"/>
		<xsl:copy>
			<xsl:for-each select = "@*">
				<xsl:copy/>
			</xsl:for-each>
			<lp:index>
				<xsl:number count = "lp:file" level = "any" format = "1"/>
			</lp:index>
			<xsl:apply-templates select = "node()"/>
		</xsl:copy>
	</xsl:template>
	<!-- This template renders an "lp:macro", which is a text or balanced XML fragment. -->
	<xsl:template match = "lp:macro">
		<xsl:call-template name = "validateMacro"/>
		<xsl:copy>
			<xsl:for-each select = "@*">
				<xsl:copy/>
			</xsl:for-each>
			<lp:index>
				<xsl:number count = "lp:macro" level = "any" format = "1"/>
			</lp:index>
			<xsl:variable name = "macroName" select = "normalize-space(lp:name)"/>
			<xsl:variable name = "macroDefinitions" select = "//lp:macro/lp:name[normalize-space(text())=$macroName]"/>
			<xsl:if test = "count($macroDefinitions) >= 2">
				<xsl:for-each select = "$macroDefinitions">
					<lp:definedInMacro>
						<xsl:number count = "lp:macro" level = "any" format = "1"/>
					</lp:definedInMacro>
				</xsl:for-each>
			</xsl:if>
			<xsl:variable name = "macroInvocations" select = "//lp:macro//lp:invoke/lp:name[normalize-space(text())=$macroName]"/>
			<xsl:for-each select = "$macroInvocations">
				<lp:invokedInMacro>
					<xsl:number count = "lp:macro" level = "any" format = "1"/>
				</lp:invokedInMacro>
			</xsl:for-each>
			<xsl:variable name = "fileInvocations" select = "//lp:file//lp:invoke/lp:name[normalize-space(text())=$macroName]"/>
			<xsl:for-each select = "$fileInvocations">
				<lp:invokedInFile>
					<xsl:number count = "lp:file" level = "any" format = "1"/>
				</lp:invokedInFile>
			</xsl:for-each>
			<xsl:apply-templates select = "node()"/>
		</xsl:copy>
	</xsl:template>
	<!--This template renders an "lp:invoke".-->
	<xsl:template match = "lp:invoke">
		<xsl:copy>
			<xsl:apply-templates/>
			<xsl:variable name = "macroName" select = "normalize-space(lp:name)"/>
			<xsl:for-each select = "//lp:macro[normalize-space(lp:name) = $macroName][1]">
				<xsl:call-template name = "validateMacro"/>
				<lp:index>
					<xsl:number count = "lp:macro" level = "any" format = "1"/>
				</lp:index>
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
</xsl:stylesheet>